<?php
/**
* © Copyright 2010 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package i2ce
* @subpackage forms
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v4.0.8
* @since v4.0.8
* @filesource 
*/ 
/** 
* Class I2CE_FormField_MAP_LINE
* 
* @access public
*/



  //stored in db as "form|id1@id2@id3|line"
class I2CE_FormField_MAP_LINE extends I2CE_FormField_MAP {

    /**
     * Componentizes the given $db_value based on component
     * @param string $db_value.  The non-componentized value
     * @param array $forms of stirng. The form names which we wish to componentize.
     * @param string $component The component we wish to encode
     * @returns string The componentized db_value
     */
    public function getComponentizedValue($db_value,$forms,$component) {
        list ($form,$id,$line) = array_pad(explode('|',$db_value,3),3,'');
        if (in_array($form,$forms)) {
            return $db_value . '@' . $component;
        } else {
            return $db_value;
        }
    }

        

    /**
     * Componentizes the given $db_value based on component
     * @param string $db_ref.  The reference to the data
     * @param array $forms of stirng. The form names which we wish to componentize.
     * @param string $component The component we wish to encode
     * @returns string The componentized db_value
     */
    public function getSQLComponentization($db_ref,$forms,$component) {
        if (count($forms) == 0) {
            return $qry;
        }
        //there are parent forms.  we need to examine this vlaue to
        //see if it is a componentized form.
        foreach ($forms as &$form) {
            $form = '\'' . mysql_real_escape_string($form)  . '\'';
        }
        //mysql substring is string,start,length
        $form_qry = 'SUBSTRING(' . $qry . ",1, LOCATE('|'," . $qry . ") - 1)";        
        $formid_qry = 'SUBSTRING(' . $qry . ",1, LENGTH(" . $qry . ") - LOCATE('|',REVERSE(" . $qry . ")) )";        
        $line_qry = 'SUBSTRING(' . $qry . ", LENGTH(" . $qry . ") - LOCATE('|',REVERSE(" . $qry . ")) + 2)";        
        $val_qry = 'IF ( ' . $form_qry . ' IN (' . implode(',',$forms) . '), CONCAT( ' . $formid_qry . ",'@" . mysql_real_escape_string($component) . "','|'," . $line_qry .")," . $qry . ')';
        //makre sure there are two occurences of | before doing the val query
        return "IF ( LOCATE('|'," . $qry . ") > 0 & LOCATE('|',SUBSTRING(" . $qry ." , LOCATE('|' , " . $qry . ")+1)) > 0 ," .   $val_qry . ',' . $qry .")";
    }


    /**
     * Return the value of this field from the database format for the given type.
     * @param mixed $value
     */
    public function getFromDB( $value ) {
        if (strpos($value,'|') !== false) {
            return array_pad(explode('|',$value,3),3,'');
        } else {
            return false;
        }
    }
    

    
    /**
     * Return the DB value for this form field.
     * @return string
     */
    public function getDBMAPValue() {
        return implode( "|", array_slice($this->getValue(),0,2) );
    }

    /**
     * Sets the value of this field from the posted form.
     * @param array $post The $_POST array holding the values for this form.
     */
    public function setFromPost( $post ) {        
        if (is_string($post)) {
            return $this->setFROMDB($post);
        }
        if (!is_array($post) || !array_key_exists('map',$post) || !is_scalar($post['map']) || !array_key_exists('line',$post) || !is_scalar($post['line'])) {
            $this->value = array("","","");
        }
        $val = $this->getFromDB( $post['map'] .'|' . $post['line']);
        if (is_array($val)) {               
            $this->value = $val;
        } else {
            $this->value = array("","","");
        }
    }

    /**
     * Return the native value for this form field.
     * @return array
     */
    public function getValue() {
        if ( !$this->isSetValue() ) {
            return array( "", "" ,"");
        }
        return $this->value;
    }


    /**
     * Get the id of the form that this field maps to
     * If not set, returns 0.
     * @returns string
     */
    public function getLine() {
        if (!$this->isSetValue()) {
            return '';
        }
        return $this->value[2];
    }

    /**
     * Checks to see if the value has been set.
     * @return boolean
     */
    public function issetValue() {
        return is_array($this->value);
    }

    
    /*
     * Checks to see if the current value for this is set and valid.
     * @return boolean
     */
    public function isValid() {
        if ( !$this->isSetValue() ) {
            return false;
        }
        print_r($this->value);
        var_dump (array(count($this->value) ,I2CE_Validate::checkString( $this->value[1] ),  $this->canSelectAnyForm() , in_array($this->value[0],$this->getSelectableForms())));
        return count($this->value) == 3 && I2CE_Validate::checkString( $this->value[1] ) 
            && ( $this->canSelectAnyForm() || in_array($this->value[0],$this->getSelectableForms()));
    }


    /**
     * Returns the value of this field as a human readable format.
     * @param I2CE_Entry $entry If a I2CE_Entry object has been passed to this method then it will return the value for that entry assuming it's an 
     * entry for this field.
     * @return mixed
     */
    public function getDisplayValue( $entry=false ) {
        if ( $entry instanceof I2CE_Entry ) {
            $value = $entry->getValue();
        } else {
            if ( !$this->isValid() ) {
                return "";
            }
            $value = $this->getValue();
        }
        return sprintf($this->getDisplayFormatString() , I2CE_List::lookup( $value[1], $value[0] ), $value[2]);
    }


    /**
     * Get the display format string
     * @param string $type.  Defaults to 'default'
     * @return string        
     */
    public function getDisplayFormatString($type = 'default') {
        $path = "meta/display/$type/format_string";
        if (!$this->optionsHasPath($path) || ! is_scalar( $format_string = $this->getOptionsByPath($path) )) {
            return "%s-%s";
        }
        return $format_string;
    }

    /**
     *get the default display style for the given type of display
     */         
    public function getDefaultDisplayStyle($type) {
        return 'list';
    }



    /**
     * Creates a drop down list of options.
     * @param I2CE_Template $template
     * @param DOMNode $node -- the node we wish to add the drop down list under
     * @pararm array $fields
     * @param array $forms  The selectable forms for this map.  Order does not matter
     * @param array $limits.  An array, indexed by form name, of limiting data
     * @param array $orders.  An array, indexed by form name, of ordering data
     * @param boolean $show_hidden.  Show the hidden members of the list, defaults to false.
     * @returns mixed DOMNode or an array of DOMNodes to add.
     */
    protected function create_DOMEditable_list($node, $template, $form_node,$fields,$forms,$limits, $orders, $show_hidden= false) {
        $list = I2CE_List::flattenDataTree( I2CE_List::createDataTree( $fields,$forms,$limits, $orders, false, ' ', $show_hidden));         
        $selected =$this->getDBMapValue();
        $selectNode = $template->createElement('select',array( 'name'=>$this->getHTMLName() . '[map]'));                
        $lineNode = $template->createElement('input',array( 'type'=>'text','name'=>$this->getHTMLName() . '[line]','value'=>$this->getLine()));                

        $this->setElement($selectNode);
        if ( $form_node->hasAttribute( "blank" ) ) {
            $blank_text = $form_node->getAttribute( "blank" );
        } else {
            $blank_text = "Select One";
        }
        $selectNode->appendChild( $template->createElement( "option", array( 'value' => '' ), $blank_text ) );
        foreach ($list as $d) {
            $attrs = array('value'=>$d['value']);
            if ($d['value'] == $selected) {
                $attrs['selected'] = 'selected';
            }            
            $selectNode->appendChild($template->createElement('option', $attrs, $d['display']));
        }
        $node->appendChild($selectNode);
        $node->appendChild($lineNode);
    }






}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
